#!/bin/python
import os

from vagd import Vagd, Qegd, Shgd, Dogd, Logd, wrapper, Box
from pwn import *

GDB_OFF = 0x555555555000
IP = ''
PORT = 0
BINARY = './bin/sysinfo'
ARGS = []
ENV = {}
API = True
GDB = f"""
b main
c"""

context.binary = exe = ELF(BINARY, checksec=False)
context.aslr = False

byt = lambda x: str(x).encode()


def vms():
    log.info("Testing Local")
    vm = Logd(exe.path)
    yield vm
    log.info("Testing Vagrant")
    vm = Vagd(exe.path, vbox=Box.VAGRANT_JAMMY64, tmp=True, fast=True, ex=True)
    yield vm
    log.info("Testing Vagrant restore")
    vm = Vagd(exe.path, vbox=Box.VAGRANT_JAMMY64, tmp=True, fast=True, ex=True)
    yield vm
    vm._v.halt()
    if os.path.exists(Dogd.LOCKFILE):
        os.remove(Dogd.LOCKFILE)
    log.info("Testing Docker for Ubuntu")
    vm = Dogd(exe.path, image=Box.DOCKER_JAMMY, tmp=True, ex=True, fast=True)
    yield vm
    log.info("Testing Docker for Ubuntu restore")
    vm = Dogd(exe.path, image=Box.DOCKER_JAMMY, tmp=True, ex=True, fast=True)
    yield vm
    vm._client.containers.get(vm._id).kill()
    os.remove(Dogd.LOCKFILE)
    log.info("Testing Docker for Alpine")
    vm = Dogd(exe.path + "_stat", image=Box.DOCKER_ALPINE_316, tmp=True, ex=True, fast=True)
    yield vm
    log.info("Testing Docker for Alpine restore")
    vm = Dogd(exe.path + "_stat", image=Box.DOCKER_ALPINE_316, tmp=True, ex=True, fast=True)
    yield vm
    vm._client.containers.get(vm._id).kill()
    log.info("Testing Qemu")
    vm = Qegd(exe.path, img=Box.QEMU_JAMMY, tmp=True, ex=True, fast=True)
    yield vm
    log.info("Testing Qemu restore")
    vm = Qegd(exe.path, img=Box.QEMU_JAMMY, tmp=True, ex=True, fast=True)
    yield vm
    log.info("Testing SSH")
    yield Shgd(exe.path, user=vm._user, port=vm._port, tmp=True, ex=True, fast=True)
    os.system('kill $(pgrep qemu)')


for vm in vms():
    t = vm.start(argv=ARGS, env=ENV, gdbscript=GDB, api=API)

    sleep(1)
    g = wrapper.GDB(t)
    g.execute('p "PWN"')
    g.execute('c')

    log.info(t.recvall().decode())
    t.close()
    os.system('tmux kill-pane')

print("Everything executed without errors")



import pwn
from typing import Iterable
from vagd.virts.pwngd import Pwngd


class Logd(Pwngd):
    """
    local execution of binary

    :param binary: binary to execute
    """

    _binary: str

    def _vm_setup(self) -> None:
        """
        NOT IMPLEMENTED
        """
        pwn.log.error("NOT IMPLEMENTED")
    def _ssh_setup(self) -> None:
        """
        NOT IMPLEMENTED
        """
        pwn.log.error("NOT IMPLEMENTED")

    def __init__(self,
                 binary: str,
                 **kwargs):
        """
        :param binary: binary to execute
        """
        self._binary = binary
    def _sync(self, file: str) -> None:
        """
        NOT IMPLEMENTED
        """
        pwn.log.error("NOT IMPLEMENTED")

    def _mount(self, remote_dir: str, local_dir: str) -> None:
        """
        NOT IMPLEMENTED
        """
        pwn.log.error("NOT IMPLEMENTED")

    def _mount_lib(self, remote_lib: str = '/usr/lib') -> None:
        """
        NOT IMPLEMENTED
        """
        pwn.log.error("NOT IMPLEMENTED")

    def system(self, cmd: str) -> pwn.tubes.ssh.ssh_channel:
        """
        NOT IMPLEMENTED
        """
        pwn.log.error("NOT IMPLEMENTED")

    def _install_packages(self, packages: Iterable):
        """
        NOT IMPLEMENTED
        """
        pwn.log.error("NOT IMPLEMENTED")

    def put(self, file: str, remote: str = None):
        """
        NOT IMPLEMENTED
        """
        pwn.log.error("NOT IMPLEMENTED")

    def debug(self, **kwargs) -> pwn.process:
        """
        run binary with gdb locally
        :param kwargs: pwntool arguments
        :rtype: pwn.process
        """
        return self.pwn_debug(**kwargs)

    def pwn_debug(self, argv: list[str] = None, **kwargs) -> pwn.process:
        """
        run binary with gdb locally
        :param argv: comandline arguments for binary
        :param kwargs: pwntool arguments
        :rtype: pwn.process
        """
        return pwn.gdb.debug([self._binary] + argv, **kwargs)

    def process(self, argv: list[str] = None, **kwargs) -> pwn.process:
        """
        run binary locally
        :param argv: comandline arguments for binary
        :param kwargs: pwntool parameters
        :return: pwntools process
        """
        return pwn.process([self._binary] + argv, **kwargs)

    def start(self,
              argv: list[str] = None,
              gdbscript: str = '',
              api: bool = None,
              **kwargs) -> pwn.process:
        """
        start binary locally and return pwn.process
        :param argv: commandline arguments for binary
        :param gdbscript: GDB script for GDB
        :param api: if GDB API should be enabled (experimental)
        :param kwargs: pwntool parameters
        :return: pwntools process, if api=True tuple with gdb api
        """
        if pwn.args.GDB:
            return self.pwn_debug(argv=argv, gdbscript=gdbscript, api=api, **kwargs)
        else:
            return self.process(argv=argv, **kwargs)



import pwn
import time
from vagd.virts.pwngd import Pwngd
class Shgd(Pwngd):
    """ 
    ssh interface for pwntools

    :param binary: binary to execute
    :param user: ssh user
    :param host: ssh hostname
    :param port: ssh port
    :param keyfile: ssh keyfile (default in .vagd)
    :param kwargs: parameters to pass through to super
    """

    DEFAULT_HOST = 'localhost'
    DEFAULT_PORT = 22
    DEFAULT_USER = 'root'

    _user: str
    _host: str
    _port: int
    _keyfile: str

    def _vm_setup(self) -> None:
        """
        pass
        """
        pass

    _TRIES = 3  # three times the charm
    def _ssh_setup(self) -> None:
        """
        setup ssh connection
        """
        for _ in range(Shgd._TRIES):
            try:
                self._ssh = pwn.ssh(
                    user=self._user,
                    host=self._host,
                    port=self._port,
                    keyfile=self._keyfile,
                    ignore_config=True
                )
                break
            except:
                if _ + 1 == Shgd._TRIES:
                    pwn.log.error('SSH failed, pls try again')
                else:
                    pwn.log.info('Trying again')
                time.sleep(15)

    def __init__(self,
                 binary: str,
                 user: str = DEFAULT_USER,
                 host: str = DEFAULT_HOST,
                 port: int = DEFAULT_PORT,
                 keyfile: str = Pwngd.KEYFILE,
                 **kwargs):
        """

        :param binary: binary to execute
        :param user: ssh user
        :param host: ssh hostname
        :param port: ssh port
        :param keyfile: ssh keyfile (default in .vagd)
        :param kwargs: parameters to pass through to super
        """
        self._user = user
        self._host = host
        self._port = port
        self._keyfile = keyfile

        self._ssh_setup()

        super().__init__(binary=binary, **kwargs)





import os
from typing import Dict

import docker
import pwn

from vagd import templates, helper
from vagd.box import Box
from vagd.virts.pwngd import Pwngd
from vagd.virts.shgd import Shgd


class Dogd(Shgd):
    """
    | Docker virtualization for pwntools

    :param binary: binary to execute
    :param image: docker base image
    :param user: name of user on docker container
    :param forward: Dictionary of forwarded ports, needs to follow docker api format: 'hostport/(tcp|udp)' : guestport
    :param ex: if experimental features, e.g. alpine, gdbserver should be enabled
    :param fast: mounts libs locally for faster symbol extraction (experimental) NOT COMPATIBLE WITH ALPINE
    :param kwargs: parameters to pass through to super

    | SSH from cmd

    .. code-block:: bash

        ssh -o "StrictHostKeyChecking=no" -i ~/.vagd/keyfile -p $(cut .vagd/docker.lock -d":" -f 2) vagd@0.0.0.0

    | connect with docker exec

    .. code-block:: bash

       docker exec -it $(cut ./.vagd/docker.lock -d":" -f 1) /bin/bash

    | Kill from cmd:

    .. code-block:: bash

        docker kill $(cut ./.vagd/docker.lock -d":" -f 1)

    | Docker containers are automatically removed after they stop
    | Docker images need to be manually removed from docker
    | Dockerfiles are stored in home directory to allow caching ~/.vagd/docker/<image>/Dockerfile

    .. code-block:: bash

        docker images # list images
        docker rmi <id> # remove correct image
    """

    _image: str
    _user: str
    _port: int
    _client: docker.client
    _id: str
    _dockerdir: str
    _dockerfile: str
    _isalpine: bool
    _gdbsrvport: int
    _ex: bool
    _forward: Dict[str, int]

    DOCKERHOME = Pwngd.HOME_DIR + "docker/"
    DEFAULT_USER = 'vagd'
    DEFAULT_PORT = 2222
    DEFAULT_IMAGE = Box.DOCKER_FOCAL

    DEFAULT_PACKAGES = Pwngd.DEFAULT_PACKAGES + ["openssh-server"]
    LOCKFILE = Pwngd.LOCAL_DIR + 'docker.lock'

    def _create_dockerfile(self):

        pwn.log.info(f'create new Dockerfile at f{self._dockerfile}')
        if not os.path.exists(Pwngd.KEYFILE):
            helper.generate_keypair()

        if not os.path.exists(self._dockerdir + "keyfile.pub"):
            os.link(Pwngd.PUBKEYFILE, self._dockerdir + "keyfile.pub")
        template = templates.DOCKER_ALPINE_TEMPLATE if self._isalpine else templates.DOCKER_TEMPLATE

        with open(self._dockerfile, 'w') as dockerfile:
            dockerfile.write(
                template.format(image=self._image,
                                packages=' '.join(Dogd.DEFAULT_PACKAGES),
                                user=self._user,
                                keyfile=os.path.basename(self._dockerdir + "keyfile.pub")))

    def _create_docker_instance(self):
        pwn.log.info('starting docker instance')
        self._port = helper.first_free_port(Dogd.DEFAULT_PORT)
        self._forward.update({'22/tcp': self._port})
        if self._isalpine:
            self._gdbsrvport = helper.first_free_port(Pwngd.STATIC_GDBSRV_PORT)
            self._forward.update({f'{self._gdbsrvport}/tcp': self._gdbsrvport})

        dir = os.path.dirname(os.path.realpath(__file__))
        with open(dir[:dir.rfind('/')] + '/res/seccomp.json', 'r') as seccomp_file:
            seccomp_rules = seccomp_file.read().strip()

        container = self._client.containers.run(self._bimage, ports=self._forward, detach=True, remove=True, security_opt=[f'seccomp:{seccomp_rules}'])
        self._id = container.id
        pwn.log.info(f'started docker instance {container.short_id}')
        with open(Dogd.LOCKFILE, 'w') as lockfile:
            lockfile.write(':'.join((container.id, str(self._port), str(self._gdbsrvport))))

    def _build_image(self):
        pwn.log.info('building docker image')
        return self._client.images.build(path=os.path.dirname(self._dockerfile), tag=f'vagd/{self._image}')[0]

    def _vm_create(self):

        if not os.path.exists(Pwngd.LOCAL_DIR):
            os.makedirs(Pwngd.LOCAL_DIR)

        if not os.path.exists(self._dockerfile):
            self._create_dockerfile()

        self._bimage = self._build_image()

        self._create_docker_instance()

    def _vm_setup(self) -> None:
        self._client = docker.from_env()
        if not os.path.exists(Dogd.LOCKFILE):
            pwn.log.info(f'No Lockfile {Dogd.LOCKFILE} found, creating new Docker Instance')
            self._vm_create()
        else:
            with open(Dogd.LOCKFILE, 'r') as lockfile:
                data = lockfile.readline().split(':')
                self._id = data[0]
                self._port = int(data[1])
                if self._isalpine:
                    self._gdbsrvport = int(data[2])
            if not self._client.containers.list(filters={'id':self._id}):
                pwn.log.info(f'Lockfile {Dogd.LOCKFILE} found, container not running, creating new one')
                self._vm_create()
            else:
                pwn.log.info(
                    f'Lockfile {Dogd.LOCKFILE} found, Docker Instance f{self._client.containers.get(self._id).short_id}')

    def __init__(self,
                 binary: str,
                 image: str = DEFAULT_IMAGE,
                 user: str = DEFAULT_USER,
                 forward: Dict[str, int] = None,
                 ex: bool = False,
                 fast: bool = False,
                 **kwargs):
        """

        :param binary: binary to execute
        :param image: docker base image
        :param user: name of user on docker container
        :param forward: Dictionary of forwarded ports, needs to follow docker api format: 'hostport/(tcp|udp)' : guestport
        :param ex: if experimental features, e.g. alpine, gdbserver should be enabled
        :param fast: mounts libs locally for faster symbol extraction (experimental) NOT COMPATIBLE WITH ALPINE
        :param kwargs: parameters to pass through to super
        """

        self._image = image
        self._isalpine = 'alpine' in image
        self._gdbsrvport = None
        self._dockerdir = Dogd.DOCKERHOME + f'{self._image}/'
        if not (os.path.exists(Dogd.DOCKERHOME) and os.path.exists(self._dockerdir)):
            os.makedirs(self._dockerdir)
        self._dockerfile = self._dockerdir + 'Dockerfile'
        self._user = user
        self._forward = forward
        self._ex = ex
        if self._isalpine and not self._ex:
            pwn.log.error("Docker alpine images requires experimental features")
        if self._forward is None:
           self._forward = dict()

        self._vm_setup()

        super().__init__(binary=binary,
                         user=self._user,
                         port=self._port,
                         ex=ex,
                         fast=fast,
                         gdbsrvport=self._gdbsrvport,
                         **kwargs)



import os
import pwn
import pathlib
from shutil import which
from abc import ABC, abstractmethod
from typing import Union, Dict, Iterable


class Pwngd(ABC):
    """
    start binary on remote and return pwn.process

    :param argv: commandline arguments for binary
    :param gdbscript: GDB script for GDB
    :param api: if GDB API should be enabled (experimental)
    :param sysroot: sysroot dir (experimental)
    :param gdb_args: extra gdb args (experimental)
    :param kwargs: pwntool parameters
    :return: pwntools process, if api=True tuple with gdb api
    """
    LOCAL_DIR = './.vagd/'
    HOME_DIR = os.path.expanduser('~/.vagd/')
    SYSROOT = LOCAL_DIR + 'sysroot/'
    SYSROOT_LIB = SYSROOT + 'lib/'
    SYSROOT_LIB_DEBUG = SYSROOT + 'lib/debug'
    KEYFILE = HOME_DIR + 'keyfile'
    PUBKEYFILE = KEYFILE + '.pub'
    DEFAULT_PORT = 2222
    STATIC_GDBSRV_PORT = 42069

    _path: str
    _gdbsrvport: int
    _binary: str
    _ssh: pwn.ssh
    _experimental: bool
    _fast: bool

    @abstractmethod
    def _vm_setup(self) -> None:
        """
        setup vagrant machine creates new one if no Vagrantfile is specified or box does not match
        """
        pass

    @abstractmethod
    def _ssh_setup(self) -> None:
        """
        setup ssh connection
        """
        pass

    def _sync(self, file: str) -> None:
        """
        upload file on remote if not exist
        :type file: file to upload
        """
        sshpath = pwn.SSHPath(os.path.basename(file))
        if not sshpath.exists():
            self.put(file)

    _SSHFS_TEMPLATE = \
        'sshfs -p {port} -o StrictHostKeyChecking=no,ro,IdentityFile={keyfile} {user}@{host}:{remote_dir} {local_dir}'

    def _mount(self, remote_dir: str, local_dir: str) -> None:
        """
        mount remote dir on local wiith sshfs
        :param remote_dir: directory on remote to mount
        :param local_dir: local mount point
        """
        if not which('sshfs'):
            pwn.log.error('sshfs isn\'t installed')
        cmd = Pwngd._SSHFS_TEMPLATE.format(port=self._ssh.port,
                                           keyfile=self._ssh.keyfile,
                                           user=self._ssh.user,
                                           host=self._ssh.host,
                                           remote_dir=remote_dir,
                                           local_dir=local_dir)
        pwn.log.info(cmd)
        os.system(cmd)

    def _mount_lib(self, remote_lib: str = '/usr/lib') -> None:
        """
        mount the lib directory of remote
        """
        if not (os.path.exists(Pwngd.SYSROOT) and os.path.exists(Pwngd.SYSROOT_LIB)):
            os.makedirs(Pwngd.SYSROOT_LIB)
        if not os.path.ismount(Pwngd.SYSROOT_LIB):
            pwn.log.info('mounting libs in sysroot')
            self._mount(remote_lib, Pwngd.SYSROOT_LIB)

    def system(self, cmd: str) -> pwn.tubes.ssh.ssh_channel:
        """
        executes command on vm, interface to  pwnlib.tubes.ssh.ssh.system

        :param cmd: command to execute on vm
        :return: returns
        """
        return self._ssh.system(cmd)

    DEFAULT_PACKAGES = ['gdbserver', 'libc6-dbg', 'python3']

    def _install_packages(self, packages: Iterable):
        """
        install packages on remote machine

        :param packages: packages to install on remote machine
        """
        self.system("sudo apt update").recvall()
        packages_str = " ".join(packages)
        self.system(f"NEEDRESTART_MODE=a sudo -E apt install -y {packages_str}").recvall()

    def put(self, file: str, remote: str = None):
        """
        upload file or vm on vm,

        :param file: file to upload
        :param remote: remote location of file, working directory if not specified
        :return: returns
        """
        if os.path.isdir(file):
            self._ssh.upload_dir(file, remote=remote)
        else:
            self._ssh.upload(file, remote=remote)

    def __init__(self,
                 binary: str,
                 files: Union[str, list[str]] = None,
                 packages: Iterable = None,
                 tmp: bool = False,
                 gdbsrvport: int = None,
                 fast: bool = False,
                 ex: bool = False):
        """
        Default init setups provided ssh machine

        :param binary: binary for VM debugging
        :param files: other files or directories that need to be uploaded to VM
        :param packages: packages to install on vm
        :param tmp: if a temporary directory should be created for files
        :param gdbsrvport: specify static gdbserver port, REQURIES port forwarding to localhost
        :param fast: mounts libs locally for faster symbol extraction (experimental)
        :param ex: if experimental features should be enabled
        """

        if packages is not None:
            self._install_packages(packages)

        self._path = binary
        self._gdbsrvport = gdbsrvport
        self._binary = './' + os.path.basename(binary)

        self._fast = fast
        self._experimental = ex

        if self._fast:
            if self._experimental:
                self._mount_lib()
            else:
                pwn.error('requires experimental features, activate with ex=True')

        pwn.context.ssh_session = self._ssh
        if tmp:
            self._ssh.set_working_directory()

        self._sync(self._path)
        self.system('chmod +x ' + self._binary)

        # Copy files to remote
        if isinstance(files, str):
            self._sync(files)
        elif hasattr(files, '__iter__'):
            for file in files:
                self._sync(file)

    def debug(self,
              argv: list[str] = None,
              exe: str = '',
              env: Dict[str, str] = None,
              ssh=None,
              gdbscript: str = '',
              api: bool = False,
              sysroot: str = None,
              gdb_args: list[str] = None,
              **kwargs) -> pwn.process:
        """
        run binary in vm with gdb and experimental features

        :param argv: command line arguments
        :param exe: exe to execute
        :param env: environment variable dictionary
        :param ssh: ignored self._ssh is used instead
        :param gdbscript: used gdbscript
        :param api: return gdb python api interface
        :param sysroot: sysroot directory
        :param gdb_args: additional gdb arguments
        :param kwargs: pwntool arguments
        :rtype: pwn.process
        """
        if argv is None:
            argv = []
        args = [self._binary, ] + argv
        if gdb_args is None:
            gdb_args = list()
        pwn.log.warn('using experimental features')
        ssh = self._ssh
        if isinstance(args, (bytes, pwn.six.text_type)):
            args = [args]

        runner = pwn.gdb._get_runner(ssh)
        which = pwn.gdb._get_which(ssh)

        args, env = pwn.gdb.misc.normalize_argv_env(args, env, pwn.log)
        if env:
            env = {bytes(k): bytes(v) for k, v in env}

        args = pwn.gdb._gdbserver_args(args=args, which=which, env=env)

        # set static port if wanted
        if self._gdbsrvport is not None:
            for i in range(len(args)):
                if args[i] == 'localhost:0':
                    args[i] = f':{self._gdbsrvport}'
                    break

        # Make sure gdbserver/qemu is installed
        if not which(args[0]):
            pwn.log.error("%s is not installed" % args[0])

        # Start gdbserver/qemu
        # (Note: We override ASLR here for the gdbserver process itself.)
        gdbserver = runner(args, env=env, aslr=1, **kwargs)

        # Set the .executable on the process object.
        gdbserver.executable = exe

        # Find what port we need to connect to
        port = pwn.gdb._gdbserver_port(gdbserver, ssh) if self._gdbsrvport is None else self._gdbsrvport

        host = '127.0.0.1'

        if self._fast:
            gdb_args += ["-ex", f"set sysroot {pathlib.Path().resolve()}/{Pwngd.SYSROOT}"]
            gdbscript = f"set debug-file-directory {Pwngd.SYSROOT_LIB_DEBUG}\n" + gdbscript
        elif sysroot:
            gdb_args += ["-ex", f"set sysroot {sysroot}"]
            gdbscript = f"set debug-file-directory ./{sysroot}/lib/debug\n" + gdbscript
        else:
            gdbscript = "set debug-file-directory /lib/debug\n" + gdbscript

        gdb_args += ["-ex", f"file -readnow {self._path}"]

        tmp = pwn.gdb.attach((host, port), exe=exe, gdbscript=gdbscript,
                             gdb_args=gdb_args, ssh=ssh, api=api)
        if api:
            _, gdb = tmp
            gdbserver.gdb = gdb

        # gdbserver outputs a message when a client connects
        garbage = gdbserver.recvline(timeout=1)

        # Some versions of gdbserver output an additional message
        garbage2 = gdbserver.recvline_startswith(b"Remote debugging from host ", timeout=2)

        return gdbserver

    def pwn_debug(self, argv: list[str] = None, ssh=None, **kwargs) -> pwn.process:
        """
        run binary in vm with gdb (pwnlib feature set)

        :param argv: comandline arguments for binary
        :param ssh: ignored self._ssh is used instead
        :param kwargs: pwntool parameters
        :return: pwntools process
        """
        if argv is None:
            argv = list()
        return pwn.gdb.debug([self._binary] + argv, ssh=self._ssh, **kwargs)

    def process(self, argv: list[str] = None, **kwargs) -> pwn.process:
        """
        run binary in vm as process

        :param argv: comandline arguments for binary
        :param kwargs: pwntool parameters
        :return: pwntools process
        """
        if argv is None:
            argv = list()
        return self._ssh.process([self._binary] + argv, **kwargs)

    def start(self,
              argv: list[str] = None,
              gdbscript: str = '',
              api: bool = None,
              sysroot: str = None,
              gdb_args: list = None,
              **kwargs) -> pwn.process:
        """
        start binary on remote and return pwn.process

        :param argv: commandline arguments for binary
        :param gdbscript: GDB script for GDB
        :param api: if GDB API should be enabled (experimental)
        :param sysroot: sysroot dir (experimental)
        :param gdb_args: extra gdb args (experimental)
        :param kwargs: pwntool parameters
        :return: pwntools process, if api=True tuple with gdb api
        """
        if pwn.args.GDB:
            if self._experimental:
                return self.debug(argv=argv, gdbscript=gdbscript, gdb_args=gdb_args, sysroot=sysroot,
                                  api=api, **kwargs)
            else:
                if gdb_args or sysroot or api:
                    pwn.error('requires experimental features, activate with ex=True in constructor')
                return self.pwn_debug(argv=argv, gdbscript=gdbscript, sysroot=sysroot, **kwargs)
        else:
            return self.process(argv=argv, **kwargs)



import os
import pwn
import stat
import typer
import importlib.metadata
from typing import Optional

DOGD = "vm = Dogd(exe.path, image=Box.DOCKER_JAMMY, ex=True, fast=True)  # Docker"
VAGD = "vm = Vagd(exe.path, vbox=Box.VAGRANT_JAMMY64, ex=True, fast=True)  # Vagrant"
QEGD = "vm = Qegd(exe.path, img=Box.QEMU_JAMMY, user='ubuntu', ex=True, fast=True)  # Qemu"
SHGD = "vm = Shgd(exe.path, user='user', port=22, ex=True, fast=True)  # SSH"

app = typer.Typer(context_settings={"help_option_names": ["-h", "--help"]})

def _version(value: bool) -> None:
    if value:
        version = importlib.metadata.version('vagd')
        typer.echo(f"VAGD v{version}")
        raise typer.Exit()
@app.callback()
def main(
        version: Optional[bool] = typer.Option(
            None,
            "--version",
            "-v",
            help="Show current vagd version and exit.",
            callback=_version,
            is_eager=True,
        ),
    ) -> None:
        pass

@app.command()
def template(
        binary: Optional[str] = typer.Argument('', help='Binary to Exploit'),
        ip: Optional[str] = typer.Argument('', help='Ip or Domain of the remote target'),
        port: Optional[int] = typer.Argument(0, help='port of the remote target'),
        output: Optional[str] = typer.Option('', '-o', help='output file of the template (also add +x), default stdout'),
        libc: Optional[str] = typer.Option('', '--libc', '-l', help='add libc to template'),
        aslr: Optional[bool] = typer.Option(False, '--aslr', '-a', help='enable gdb ASLR (default: disabled for gdb)'),
        dogd: Optional[bool] = typer.Option(False, '--dogd', '--docker', '-d', help='create docker template'),
        qegd: Optional[bool] = typer.Option(False, '--qegd', '--qemu', '-q', help='create qemu template'),
        vagd: Optional[bool] = typer.Option(False, '--vagd', '--vagrant', '-v', help='create vagrant template'),
        shgd: Optional[bool] = typer.Option(False, '--shgd', '--ssh', '-s', help='create ssh template'),
        local: Optional[bool] = typer.Option(False, '--local', help='create local template'),
):
    """
    creates a template
    """
    templatePath = os.path.dirname(os.path.realpath(__file__))
    templateChunks = []
    templatePath += '/res/local_template.txt' if local else '/res/template.txt'
    multi = False
    if not any((dogd, qegd, vagd, shgd)):
        dogd = qegd = vagd = shgd = True
        multi = True

    dependencies = []
    vms = []
    if dogd:
        dependencies.append('Dogd')
        vms.append(DOGD)
    if qegd:
        dependencies.append('Qegd')
        vms.append(('# ' if multi else '') + QEGD)
    if vagd:
        dependencies.append('Vagd')
        vms.append(('# ' if multi else '') + VAGD)
    if shgd:
        dependencies.append('Shgd')
        vms.append(('# ' if multi else '') + SHGD)

    with open(templatePath, 'r') as templateFile:
        for line in templateFile.readlines():

            if not libc and line.startswith('libc'):
                continue
            templateChunks.append(line)

        template = ''.join(templateChunks).format('{}',
                                                  binary=binary,
                                                  ip=ip,
                                                  port=str(port),
                                                  libc=libc,
                                                  aslr=aslr,
                                                  dependencies=', '.join(dependencies),
                                                  vms=('\n' + ' ' * 8).join(vms))

        if output:
            with open(output, 'w') as exploitFile:
                exploitFile.write(template)
            current_permissions = os.stat(output).st_mode
            new_permissions = current_permissions | (stat.S_IXUSR)
            os.chmod(output, new_permissions)
        else:
            typer.echo(template)
@app.command()
def info(
        binary: str = typer.Argument(..., help='Binary to analyse'),
    ):
    """
    analyses the binary, prints checksec and .comment (often includes Distro and Compiler info)
    """
    elf = pwn.ELF(binary)
    pwn.log.info(elf.section('.comment').decode().replace('\0', '\n'))


def start():
    app()



from collections.abc import Callable, Iterable

import gdb
from gdb import _PrettyPrinterLookupFunction

class PrettyPrinter:
    name: str
    subprinters: list[SubPrettyPrinter] | None
    enabled: bool

    def __init__(self, name: str, subprinters: Iterable[SubPrettyPrinter] | None = ...) -> None: ...
    def __call__(self, val: gdb.Value) -> gdb._PrettyPrinter | None: ...

class SubPrettyPrinter:
    name: str
    enabled: bool

    def __init__(self, name: str) -> None: ...

class RegexpCollectionPrettyPrinter(PrettyPrinter):
    def __init__(self, name: str) -> None: ...
    def add_printer(self, name: str, regexp: str, gen_printer: _PrettyPrinterLookupFunction) -> None: ...

class FlagEnumerationPrinter(PrettyPrinter):
    def __init__(self, enum_type: str) -> None: ...

def register_pretty_printer(
    obj: gdb.Objfile | gdb.Progspace | None,
    printer: PrettyPrinter | Callable[[gdb.Value], gdb._PrettyPrinter | None],
    replace: bool = ...,
) -> None: ...
